/*
This algorithm was posted on musicdsp.org by andy[DOT]rossol[AT]bluewin[DOT]ch
I've changed some code.
The original code can be found here:
http://musicdsp.org/archive.php?classid=3#135
*/
#include <iostream>

using namespace std;


class CDCFilter
{
public:
	CDCFilter()
	{
		cout << "CDCFilter()  default constructor: Don't forget to call 'init()' before processing samples. \n";
	}

	CDCFilter( unsigned int sampleRate )
	{
		init( sampleRate );
	}

	void init( unsigned int sampleRate )
	{
		/*
			(-3dB @ 40Hz): R = 1-(250/samplerate)
			(-3dB @ 30Hz): R = 1-(190/samplerate)
			(-3dB @ 20Hz): R = 1-(126/samplerate)
		*/
		R = 1.0f - ( 250.0f / (float)sampleRate);
		xn_min1 = 0.f;
		yn_min1 = 0.f;
	}

	void reset() {
		xn_min1 = 0.f;
		yn_min1 = 0.f;
	}

	void processBlock( float * buffer, unsigned int size )
	{
		for  (unsigned int i = 0; i < size; i++)
		{
			buffer[i] = process( buffer[i] );
		}
	}
	
	inline float process(float x)
	{
		yn_min1 = x - xn_min1 + R * yn_min1;
		xn_min1 = x;
		return yn_min1;
	}
	
private:
	float yn_min1, xn_min1, R;
};

/*
y(n) = x(n) - x(n-1) + R * y(n-1) 
// "R" between 0.9 .. 1
// n=current (n-1)=previous in/out value
...
"R" depends on sampling rate and the low frequency point. Do not set "R" to a fixed value (e.g. 0.99) if you don't know the sample rate. Instead set R to:
(-3dB @ 40Hz): R = 1-(250/samplerate)
(-3dB @ 30Hz): R = 1-(190/samplerate)
(-3dB @ 20Hz): R = 1-(126/samplerate)
 */
