/*
	This algorithm was posted on musicdsp.org by zioguido@gmail.com	
	I've changed some code.
	The original code can be found here:
	http://musicdsp.org/archive.php?classid=3#257
*/

#include <cmath>
#include <iostream>


class CParamSmooth
{
public:
	CParamSmooth()
	{
		std::cout << "CParamSmooth default constructor: don't forget to call 'init' before use." << std::endl;
	}
    CParamSmooth( float smoothingTimeInMs, float samplingRate )
    {
		init( smoothingTimeInMs, samplingRate);
    }

	void init(float smoothingTimeInMs, float samplingRate )
	{
		const float c_twoPi = 6.283185307179586476925286766559f;

		a = exp(-c_twoPi / (smoothingTimeInMs * 0.001f * samplingRate));
		b = 1.f - a;
		z = 0.f;
		resetZ = 0.f;
	}
	
	void setZ(float x)
	{
		z = x;
		resetZ = x;
	}
	
    ~CParamSmooth()
    {
        
    }
	
	void reset()
	{
		z = resetZ;
	}
	
	inline float process(float in)
	{
		return z = (in * b) + (z * a);
	}

	void processBlock(float buf[], unsigned int size)
	{
		for (unsigned int i = 0; i < size; i++)
		{
			buf[i] = CParamSmooth::process( buf[i] );
		}
	}
	
private:
	float a;
    float b;
    float z;

	float resetZ;
};

