#pragma once
#include <iostream>
#include <cassert>

using namespace std;

/*
CRingbufGUI is a class intended to store the averaged samples from a window of audio samples.
A AverageFilter4GUI object calculates the average of audiosamples in a window, and writes them to a CRingbufGUI object.
This is not thread-save...:
A 'WaveFormComponent' will be calling 'addValueAndIncreasePos()' a lot.
DON'T call 'setSize()' while 'addValueAndIncreasePos()' can be called from another thread.
*/

class CRingbufGUI
{
public:
	CRingbufGUI() :_buffer(NULL) {}
	CRingbufGUI( CRingbufGUI & ) = delete;
	CRingbufGUI( unsigned int size )
		:
		_size(size),
		_pos(0),
		_buffer( new float[size]() )
	{

	}

	// setSize. Doesn't copy values from old array to new array. _pos = 0
	void setSize( unsigned int size )
	{
		if (_buffer) delete[] _buffer;
		_buffer = new float[size]();// '()' will initialize all elements with 0.f. 
		_size = size;
		_pos = 0;
	}

	unsigned int getSize() const
	{
		return _size;
	}

	void reset()
	{
		_pos = 0;
	}

	// Set all elements to 0.f
	void clear()
	{
		for (unsigned int i = 0; i < _size; i++)
		{
			_buffer[i] = 0.f;
		}
	}

	void writeValueAndIncreasePos( float value )
	{
		_buffer[_pos] = value;
		_pos = (_pos + 1) % _size;
	}

	float getValue(unsigned int index) const
	{
		assert( index < _size && index >= 0 );
		return _buffer[(index + _pos) % _size];
	}

	void print() const
	{
		for (unsigned int i = 0; i < _size; i++)
		{
			cout << "index:\t" << i << "\tvalue:\t" << _buffer[i] << endl;
		}
	}

	~CRingbufGUI()
	{
		if (_buffer) delete[] _buffer;
	}

private:
	unsigned int  _pos;
	unsigned int _size;
	float * _buffer;
};
