#include "CWaveformData4GUI.h"

CWaveformData4GUI::CWaveformData4GUI( unsigned int numSamples, unsigned int numPixels )
{
	init(numSamples, numPixels);
}

void CWaveformData4GUI::init( unsigned int numSamples, unsigned int numPixels )
{
	assert( numSamples > numPixels );// It doesn't make sense to have less samples than pixels.
	assert( numPixels > 0 ); // 0 pixels doesn't make sense.
	assert( numSamples > 0 );
	// assert(numPixels == _ringbufGUI.getSize());
	
	_ringbufGUI.setSize( numPixels );
	windowSize = (float)numSamples / (float)numPixels;
	reset();
	windowCountdown = windowSize;
	sum = 0.f;
}

void CWaveformData4GUI::reset()
{
	sum = 0.f;
	windowCountdown = windowSize;

	_ringbufGUI.clear();
	// '_ringbufGUI' is directly 'reset' in the gui.
	//_ringbufGUI.clear();// Sets all elements in the ringbufGUI to 0.f.
	//_ringbufGUI.reset();// Sets '_pos' to 0.
}


float * CWaveformData4GUI::makeBufferFabs( float const * inputBuf, int size )
{
	// Create new buf.
	// Don't forget to delete[] when not needed anymore.
	float * buf = new float[size];

	for (int i = 0; i < size; i++)
	{
		buf[i] = fabs( inputBuf[i] );
	}
	return buf;// Don't forget to delete[] when not needed anymore.
}

void CWaveformData4GUI::process( float const buffer[], int size )
{
	float * temp = CWaveformData4GUI::makeBufferFabs( buffer, size );
	// CWaveformData4GUI::processRecursion(temp, size);
	CWaveformData4GUI::processNoRecursion( temp, size );
	delete[] temp;
}

void CWaveformData4GUI::processRecursion( float samples[], int size )
{
	//If there are enough samples to fill current window (including transition sample).
	if ( (float)size > windowCountdown )
	{
		int numWholeSamples = (int)windowCountdown;
		for (int i = 0; i < numWholeSamples; i++)
		{
			sum += samples[i];
		}
		windowCountdown -= (float)numWholeSamples;
		
		//'samples[numWholeSamples]' is the sample where the current window ends, and the next window begins. It also calculates the average of samples in the current window.
		CWaveformData4GUI::windowTransitionSample( samples[numWholeSamples], windowCountdown );

		//Recursion: call this function again to process the remaining samples.
		//'samples' is a pointer to the first element of the array with samples. Note the pointer arithmetic: samples + numWholeSamples. Or should I write: '&samples[numWholeSamples]' ?
		CWaveformData4GUI::processRecursion( samples + numWholeSamples, size - numWholeSamples );
		//Might cause a stack overflow if recursion is to deep.
		//In the context of my program, this should be no problem. (with 'size' typicaly being between 32 and 512, and ' windowSize'  about 250 or 750.
	}
	else//If there are NOT enough samples to fill a window.
	{
		for (int i = 0; i < size; i++)
		{
			sum += samples[i];
		}
		windowCountdown -= (float)size;
	}
}

void CWaveformData4GUI::processNoRecursion( float samples[], int size )
{
	while ( size > 0 )
	{
		//cout << "size:" << size << endl;

		//If there are enough samples for current window. ( plus transitionSample, hence the 'ceil()' )
		if ( size >= ceil( windowCountdown ) )
		{
			int numWholeSamples = (int)windowCountdown;
			for (int i = 0; i < numWholeSamples; i++)
			{
				sum += samples[i];
			}
			windowCountdown -= (float)numWholeSamples;
			
			CWaveformData4GUI::windowTransitionSample( samples[numWholeSamples], windowCountdown );

			size -= numWholeSamples + 1;
			
			//Pointer arithmetic:
			samples += numWholeSamples;
		}
		else//If there are NOT enough samples for a window.
		{
			for (int i = 0; i < size; i++)
			{
				sum += samples[i];
			}
			windowCountdown -= (float)size;
			size = 0;// Would break do the same ?
		}
	}
}

//The 'windowTransitionSample' is the sample where the current window ends, and the next window begins.
//In between: the average of the current window fill be calculated, and written to the '_guiLinesRingbuf'.
//Then we move on to the next window.
void CWaveformData4GUI::windowTransitionSample( float sample, float fraction )
{
	//cout << fraction << endl;
	assert( fraction >= 0.f && fraction <= 1.f );
	
	//The first fraction of the 'sample' is used for the current window.
	sum += sample *  fraction;
	
	//Now the sum of the current window is complete.

	//Here the average of the samples in current window is calculated.
	float average = float( sum / windowSize );
	
	//cout << "average:\t" << average << endl;
	
	//Write 'average' to the gui-ringbuf. (CRingbufGUI is used to draw waveform).
	_ringbufGUI.writeValueAndIncreasePos( average );

	sum = 0.f;
	windowCountdown = windowSize;
	
	//Now: on to the next window.

	//The second part of 'sample' is used for the next window.
	fraction = (1.f - fraction);
	sum += sample * fraction;
	windowCountdown -= fraction;
}


