#pragma once

//#define NDEBUG

#include <cmath>
#include <cassert>
#include <iostream>

#include "CRingbufGUI.h"

using namespace std;
/*
Example:
If you wan't to draw the waveform of 44100 samples to a GUI with just 200 pixels of width ( or height),
there are a lot more samples than pixel rows (or columns).

'AverageFilter4GUI' is able to sum a number of samples ( in a window ), and calculate the average of those samples,
for a one-pixel line of waveform. The size of a window is: numSamples / numPixels

In this hypothetical case: 44100 / 200 = 220.5 samples are summed and then divided by the number of summed samples.
So the average of 220.5 samples is calculated and can be used to draw a one-pixel line of waveform.

With the CWaveformData4GUI::process(...) you can add samples. And if there are enough samples for a window, the average will be calculated.

Note: In the code I use the word 'window' often. By 'window' I mean: the collection of samples to calculate the average from.

I'm worried about precision of 'float windowSize'. 
Maybe after a while it drifts out of sync due to lack of precision. Test this !
*/

class CWaveformData4GUI
{
public:
	CWaveformData4GUI()
	{
		cout << "'CWaveformData4GUI' default constructor. Don't forget to call 'init'." << endl;
	}
	// Window size = numSamples / numPixels.
	CWaveformData4GUI( unsigned int numSamples, unsigned int numPixels );

	void init( unsigned int numSamples, unsigned int numPixels );

	// Add a block of samples. Samples are made positive first with 'fabs()'.
	void process( float const buffer[], int size );

	// Reset to initial condition and clears the ringbuf. So we can start all over again.
	void reset();

	// Get the ringbuf that contains the averages of the samples.
	inline CRingbufGUI & getRingbufGUI()
	{
		return _ringbufGUI;
	}

private: 

	// Assumes 1 channel audio.
	// Note: WARNING ! 'processRecursion' can cause stack-overflow when used with large array size !
	void processRecursion(float samples[], int size);
	void processNoRecursion(float samples[], int size);

	// Returns a dynamicaly allocated buffer. delete[] this buffer when ur finished with it !!
	float * makeBufferFabs(float const * buf, int size);

	//The 'windowTransitionSample' is the sample where the current window ends, and the next begins.
	//In between: the average of the current window will be calculated, and send to the rinbuffer pointed to by 'crbg'
	//Then 'reset()'is called.
	void windowTransitionSample(float sample, float index);

	// Stores the averages. Can by obtained via 'getRingbufGUI()' For drawing the pixel lines in the GUI.
	CRingbufGUI _ringbufGUI;

	float windowSize;//Doesn't have to be an integer value.
	float windowCountdown;//The number of samples needed till the end of current window.

	double sum;// The sum of samples in a window. Is later diveded by 'windowSize' to calculate average.
};
