/*
  ==============================================================================

    PlayHeadComponent.h
    Created: 12 Jul 2016 2:39:49pm
    Author:  illdoc

  ==============================================================================
*/

#ifndef PLAYHEADCOMPONENT_H_INCLUDED
#define PLAYHEADCOMPONENT_H_INCLUDED

// #include "../JuceLibraryCode/JuceHeader.h"

//==============================================================================
/*
	This PlayHeadComponent class is to draw a playhead. Normally a playhead moves over audio.
	The playhead is a horizontal line in the middle of ScratchComponent. (the parent component)
	In this case the playhead stays on the same spot and the audio moves beneath it.
	In analogy to a vinyl record player.

	It has 2 possible colours:
	-Colour::limegreen for play
	-Colour::firebrick for record.
	The default is firebrick for record.
*/
class PlayHeadComponent    : public Component
{
public:
    PlayHeadComponent()
    {
        // In your constructor, you should add any child components, and
        // initialise any special settings that your component needs.
		
        // mouseClicks on this component will be passed to the component below the mouseClick
        Component::setInterceptsMouseClicks(false, false);
		
		//recColour is default
		setRecColour();
    }

    ~PlayHeadComponent()
    {
    }
    
    void setPlayColour()
	{
		colour = Colours::limegreen;
		Component::repaint();
	}
	
	void setRecColour()
	{
		colour = Colours::firebrick;
		Component::repaint();
	}

    void paint (Graphics& g) override
    {
		//'colour' is Colour::limegreen or Colour::firebrick.
        g.setColour (colour);
        
        //Draw a horizontal line in the middle of the parents bounds.
        //In record-mode, this is where new samples appear.
        //In play-mode the samples under the playhead are 
		g.fillRect(0, Component::getHeight() / 2, Component::getWidth(), 3);
	}

    void resized() override
    {
        // This method is where you should set the bounds of any child
        // components that your component contains..

    }


private:
	Colour colour;
    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (PlayHeadComponent)
};


#endif  // PLAYHEADCOMPONENT_H_INCLUDED
