/*
  ==============================================================================

    This file was auto-generated!

  ==============================================================================
*/

#ifndef MAINCOMPONENT_H_INCLUDED
#define MAINCOMPONENT_H_INCLUDED

#include "../JuceLibraryCode/JuceHeader.h"

#include "WaveFormComponent.h"
#include "PlayHeadComponent.h"


//==============================================================================
/*
    This component lives inside our window, and this is where you should put all
    your controls and content.
*/
class ScratchComponent   : public Component, public Timer
{
public:
	//==============================================================================
	ScratchComponent() = delete;
	ScratchComponent( ScratchComponent & ) = delete;

	/*	Objects of 'ScratchComponent' only exist, while there is an editor.
		RingBufGUI contains averaged sample values, that can be drawn by 'waveForm'.
		A CRingBufGUI is also filled while there is no editor. */
	ScratchComponent(
		CRingbufGUI & ringbufL,
		CRingbufGUI & ringbufR,
		AudioProcessorParameter & scratchPosParam,
		AudioProcessorParameter & mouseDownParam// Determines the mode. scratch-mode or fill-mode.
	);
	
	~ScratchComponent();

    // Just paints the background for this component.
	void paint (Graphics&) override;
    
	// Resize child components.
	void resized() override;

	// Timer only runs when the mouseButton is up.
	// When the mouseButton is up, we are in fill mode, 
	// and this timer periodically paints the waveForm,
	// to show the content of GuiRingbufs. 
	// It also makes the GUI respond to automation.
	void timerCallback() override;


	/*
		While the mouse-button is down, we are in scratchMode.
		In scratchMode the ringbuf in 'waveForm' shouldn't change. 
		In scratchMode the 'waveForm' only needs to be redrawn, when it is dragged.
	*/
	virtual void mouseDown( const MouseEvent & ) override;
	virtual void mouseDrag( const MouseEvent & ) override;
	virtual void mouseUp( const MouseEvent & ) override;

private:
	// These are calculated once in the constructor.
	int halfHeight, halfWidth;
	//int height;
	
	// We're placing the 'waveFormR' one pixel extra to the right. So we get a nice separation line between the two channels.
	int halfWidthPlus1;

	// The time interval in miliseconds between calls to 'timerCallback()' in 'waveForm'
	int timerIntervalMs;
	
	//==============================================================================
	//My Child Components:
	
	// Draws a waveform based on a 'RingbufGUI' object.
	WaveFormComponent waveFormL;
	WaveFormComponent waveFormR;

	// Draws a horizontal line in the middle of this Component. 
	// This line is red in fill-mode:"recording" and green in scratch-mode: "playing"
	PlayHeadComponent playHead;

	//================================================================================
	// Reference to the audioProcessorParameters.
	AudioProcessorParameter & scratchPosParam;
	AudioProcessorParameter & mouseDownParam;
	
	JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(ScratchComponent)
};

#endif  // MAINCOMPONENT_H_INCLUDED
