/*
  ==============================================================================

    ScratchComponent.cpp
    Created: 7 Jul 2016 12:23:21pm
    Author:  illdoc

  ==============================================================================
*/

//#include <cstdlib>//just for rand()
#include "WaveFormComponent.h"


//==============================================================================
WaveFormComponent::WaveFormComponent( CRingbufGUI & ringbuf )
:ringbuf(ringbuf)
{
    // In your constructor, you should add any child components, and
    // initialise any special settings that your component needs.
	
	//Does not intercept mouse-clicks. So mouse clicks will go to Component beneath this one.
	Component::setInterceptsMouseClicks(false, false);
}

WaveFormComponent::~WaveFormComponent()
{
}




void WaveFormComponent::paint( Graphics & g )
{
    g.fillAll (Colours::black);   // Clear the background
	g.setColour (Colours::blue);

	int height = Component::getHeight();
	int width = Component::getWidth();
	
	// Draw a vertical line in the middle of the WaveForm. For when there is no sound.
	// Because if the value in the ringbuf is 0, in results in 0 when multiplied with width.
	g.drawRect( width / 2, 0, 1, height);

	// float * lines = ringbuf.getAllValues(); // Can only be used in scratch-mode.
	
	// Draw ringbuffer content. 'ringbuf' values are 0.f, 1.f or something in between.
	// 'y' iterates all horizontal lines.
	for ( int y = 0; y < height; y++ )
	{
		//The width of a line in the waveform.
		int lineWidth = int( ringbuf.getValue( y ) * (float)width );
		// int lineWidth = int( lines[y] * (float)width );

		// '( width - lineWidth )' is the part of a horizontal line that is not a waveform line.
		// We need half of that, for the x part of the top-left point of the rect.
		// NOTE ! Maybe it is more efficient to calculate 'x' before putting a value in the ringbuf...(what about resize ?)
		// 'x' is the left coordinate of the line to be drawn.
		// So 'x' is used to "center" the line.
		int x = ( width - lineWidth ) / 2;

		// Draw the lines the waveform-line.
		g.drawRect( x, y, lineWidth, 1 );
		
		// uses anti-aliasing when necessary.
		// g.drawHorizontalLine(y, x, x + lineWidth );
	}
}

void WaveFormComponent::resized()
{
	//Test this ! 
	//ringbuf.setSize(Component::getHeight());
    
	// This method is where you should set the bounds of any child
    // components that your component contains..
}
